/**********************************************************************
 * Copyright (c) 2007 IBM IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * IBM - Initial API and implementation
 **********************************************************************/
/**
 * @author slavescu
 */
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

public class DownloadManifestFiles {
	public static byte[] buffer = new byte[2 * 1024 * 1024];

	public static final void copyInputStream(InputStream in, OutputStream out)
			throws IOException {
		int len;
		while ((len = in.read(buffer)) >= 0)
			out.write(buffer, 0, len);
		out.close();
	}

	public static final void main(String[] args) {
		if (args.length != 3) {
			System.err
					.println("Usage: DownloadManifestFiles targetDir zipfileURL startEntryPrefix");
			System.err
			.println("Sample: DownloadManifestFiles d:\\marius\\test\\europa\\manifests\\ajdt\\ \"http://fullmoon.torolab.ibm.com/tools/ajdt/33/update/ajdt_1.5M4_for_eclipse_3.3M4.zip\" \"\"");
			
			return;
		}
		try {
			String prefixDir = args[0];
			String startEntry = args[2];
			boolean start = false;
			if (startEntry.equals("\\")) {
				start = true;
			}
			URL url = new URL(args[1]);
			URLConnection connection = (url).openConnection();
			InputStream inputStream = connection.getInputStream();
			System.out.println("DownloadManifestFiles.main() path="+prefixDir+url.getFile()+"\\");
			
			findAndExtractManifestFiles(prefixDir+url.getFile()+"\\", startEntry, start,
					inputStream);
			inputStream.close();
		} catch (Exception ioe) {
			ioe.printStackTrace();
			return;
		}
	}

	private static void findAndExtractManifestFiles(String prefixDir,
			String startEntry, boolean start, InputStream inputStream)
			throws IOException, FileNotFoundException {
		String fileName;
		File file, parent;
		ZipInputStream zipInputStream = new ZipInputStream(inputStream);
		while (zipInputStream.available() > -1) {
			ZipEntry entry = (ZipEntry) zipInputStream.getNextEntry();
			if (entry == null) {
				System.out.println("End of zip file !");
				break;
			}
			fileName = entry.getName();
			if (!start && fileName.startsWith(startEntry)) {
				start = true;
				System.out.println("Start extracting: " + fileName);
			}
			if (start) {
				if (fileName.toLowerCase().endsWith("manifest.mf")) {
					file = (new File(prefixDir + fileName));
					if (entry.isDirectory()) {
						file.mkdir();
						continue;
					}
					parent = file.getParentFile();
					if (parent != null && !parent.exists())
						parent.mkdirs();
					System.out.println("Extracting: " + file.getAbsolutePath());
					copyInputStream(zipInputStream, new BufferedOutputStream(
							new FileOutputStream(file)));
				} else if (fileName.toLowerCase().endsWith(".jar")) {
					System.out.println("Drill into: " + fileName);
					findAndExtractManifestFiles(prefixDir+fileName.replace('.', '_')+'\\',null,true,zipInputStream);
				}
			}
			zipInputStream.closeEntry();
		}
		if(startEntry!=null)
			zipInputStream.close();
	}
}